/***************************************************************************
                          notes.h  -  description
                             -------------------
    begin                : sob lis 16 2002
    copyright            : (C) 2002 by Micha Rudolf
    email                : mrudolf@mercury.ci.uw.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef KNOWIT_NOTES_H
#define KNOWIT_NOTES_H

#include <qstring.h>
#include <qpixmap.h>
#include <qptrdict.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include <klistview.h>
#include <kurl.h>

class TNote;

class TNoteLink
{
public:
   enum {DescriptionLink, LinkDescription, LinkOnly, DescriptionOnly};
   QString link;
   QString description;
   TNoteLink()   {}
   TNoteLink(const QString& s)   {link = s;}
   TNoteLink(const QString& s1, const QString& s2) {link = s1; description = s2;}
   int operator==(const TNoteLink& N) const   {return link == N.link &&
      description == N.description;}
   void save(QTextStream& ts) const;
   void open() const;
   void openWith() const;
   QString text(int fmt = DescriptionLink) const;
   QPixmap icon() const;
};


class TNotesCollection : public QPtrDict<TNote> {
public:
   QPixmap Pixmaps[4];
   TNotesCollection();
   ~TNotesCollection();
   void addNote(QListViewItem* item);
   void addNote(QListViewItem* item, const QString& s);
   void removeNote(QListViewItem* item);
   bool changeNote(QListViewItem* item, const QString& s);
   void clearNotes();
   void updateNotesView();
   bool modified;
   bool autoUpdate;
   const QString& text(QListViewItem* item);
};


class TNote {
   QValueList<TNoteLink> links;
   TNotesCollection* collection;
public:
   typedef enum {Empty, Text, EmptyParent, TextParent} State;
   enum {SaveChildren = 1, SaveSiblings = 2};
   QListViewItem* item;
   QString text;
   bool isEmpty();
   bool contains(const QString& sought, bool case_sensitive = false);
   bool change(const QString& s);
   void destroy();
   void updateView();
   State state();
   void open(QTextStream& ts);
   void save(QTextStream& ts, bool current = false);
   bool saveHTML(const KURL& fname, int flags = SaveChildren | SaveSiblings);
   bool saveHTMLBuf(QTextStream& ts, int level = 1,
      int flags = SaveChildren | SaveSiblings);
   void addLink(const QString& s);
   void addLink(const QString& s, const QString& d);
   int linkCount() const;
   const TNoteLink& link(int i) const;
   void removeLink(const QString& s);
   void modifyLink(int i, const QString& s);
   void modifyLinkDescription(int i, const QString& s);
   TNote(QListViewItem* i, TNotesCollection* coll);
   TNote(QListViewItem* i, TNotesCollection* coll, const QString& s);
   ~TNote();
};
#endif

